
Inventory allocation belongs to the family of optimization problems in the supply chain, and therefore in this Dataiku Solution, we decided to implement the Mixed Integer Programming (MIP) method for finding the optimal results. MIP allows us to define hard constraints that should not be violated while it is searching for an optimal solution that minimizes costs.   Furthermore, some benefits of this methodology in comparison to other optimization techniques are the following:

- straightforward implementation of business decisions through hard constraints
-  greater ability to scale it for larger dimensional problem
- probability of finding global solution
- insights about lower bounds 
- insights about the infeasibility of the problem setup

Objective goal of inventory allocation in our case includes minimisation problem of shipping and procurement costs. Moreover, the implementation of inventory allocation includes the following constraints:

- final level of inventory must be enough to fulfill demand
- level of inventory must not exceed warehouses capacities
- current shortage of inventory must be procured

# Formulation of the problem 

In the following section, we provide a mathematical expression of the inventory allocation problem as following:

```math
\begin{aligned}
\underset{x}{\text{minimize}}\quad &\sum_{i}^{N}\sum_{j}^{M}\sum_{k}^{K}x_{i,j,k} \cdot n_{k} \cdot shipping_{i,j} + \sum_{i}^{N}\sum_{r}^{R}y_{i,r} \cdot procurement_{i,r}\\
\text{subject to}\quad &\\
& \sum_{i}^{N}x_{i,j,k}\geq d_{j,k}, \quad \forall (j,k) \\
& \sum_{j}^{N}\sum_{k’}x_{i,j,k’} \cdot n_{k’,r} - q_{i,r} \leq y_{i,r}\quad \forall r,i \\
& q_{i,r} - \sum_{j}^{M}\sum_{k’}x_{i,j,k’} \cdot n_{k’,r}\leq v_{i,r}\quad \forall r,i \\
& \sum_{j}^{N}\sum_{k}^{K}x_{i,j,k} \cdot n_{k}+\sum_{r}^{R}v_{i,r}\leq capacity_{i}\quad \forall i \\
& x_{i,j,k}\quad, \quad y_{i,r}\quad, \quad v_{i,r} \in \mathbb{N}
\end{aligned}
```


$`x_{i,j,k}`$ - decision variable, stands for a number of product sets $`k`$ that will be delivered from warehouse $`i`$ to store $`j`$
$`y_{i,r}`$ - decision variable, stands for number of procurement skus $`r`$ in distributive center $`i`$
$`v_{i,r}`$ - variables that represent number of unused SKUs $`r`$ that is going to stored in warehouse $`i`$ after delivery
$`capacity_{i}`$ - capacity of warehouse $`i`$
$`shipping_{i,j}`$- cost of shipping SKU unit from warehouse $`i`$ to warehouse $`j`$
$`procurement_{i,r}`$ - procurement cost of SKU $`r`$ in warehouse $`i`$
$`d_{j,k}`$ - demand of product sets $`k`$ in store $`j`$
$`q_{i,r}`$ - current quantity of SKU $`r`$ in warehouse $`i`$
$`n_{k,r}`$ - total number of SKU $`r`$ in product sets $`k`$
$`i`$ - index of warehouse
$`j`$ - index of store
$`k`$- index of a product set
$`r`$- index of a SKU



