This section details all the requirements needed to use this solution.

# Instance Requirements
This solution is only compatible with  **Dataiku V11.2+** instances.

## Code Environment

For the code environment, please create a Python 3.6 env called **sol_inventory-allocation-optimization** with the following packages installed: 

```
pandas==1.0.5
numpy==1.19.5
absl-py==1.4.0
ortools==9.4.1874
protobuf==3.19.6
python-dateutil==2.8.1
pytz==2020.5
PyYAML==6.0
six==1.16.0
```

Code implementation for inventory allocation optimization is stored on the following [link](https://github.com/griddynamics/rnd-inventory-optimization-dataiku/tree/main), and the code itself is developed to support two different ways of usage, that is as a python package or a script. The code can also be found in the library section of this Dataiku project.

---

# Data sources requirements
Some data sources are mandatory in order to use the solution, while others will be optional. Optional data sources are needed in case of the benchmark against optimal allocation comparison.


## Mandatory data sources


### Supply nodes
[This data source](dataset:supply_nodes) is the  list of present warehouses, that is supply nodes, with their respective capacity

**Data format requirement**: 1 row =  The information of:
- Mandatory columns:
  - ***warehouse_id***  
  - ***location_capacity***  

### Supply
[This dataset](dataset:supply) contains information about current product stock at different warehouses. Also contains information about the cost of procurement per product unit at different warehouses.

**Data format requirement**: 1 row =  The information of:

- Mandatory columns:
  - ***warehouse_id***  
  - ***product_id***
  - ***product_stock_in_warehouses***
  - ***product_unit_procurement_cost***

### Demand forecast
[This dataset](dataset:demand_forecast) contains a list of product sets demand per demanding nodes

**Data format requirement** : 1 row =  The information of :

- Mandatory columns:
  - ***forecast_granularity***  
  - ***product_id*** or ***product_set_id***  if you are using product sets
     - if product sets are used, then they should be written in an explicit string format, for example: _[FOODS_2_435, FOODS_3_205]_ , _[HOUSHOLD_2_178, HOUSHOLD_2_859]_, etc
  - ***forecasted_demand***
  
  - Optional columns from Dataiku Demand Foraster solutions: 
  - ***reference_period_start_date***  
  - ***reference_period_start_date***
  - ***forecast_period_start_date**

###  Shipping
[This dataset](dataset:shipping) contains information about shipping costs between warehouses and stores for an individual unit of product.

**Data format requirement** : 1 row =  The information of :

- Mandatory columns:
  - ***warehouse_id***
  - ***forecast_granularity***  
  - ***product_shipping_cost***
  
### Demand nodes
[This dataset](dataset:demand_nodes) contains the list of demand nodes.

**Data format requirement** : 1 row =  The information of :

- Mandatory columns:
  - ***forecast_granularity***  

## Optional data sources


### Product procurement baseline
[This dataset](dataset:product_procurement_baseline) contains information about the number of procured products at different warehouses. This data represents baseline numbers for procurement quantities.

**Data format requirement** : 1 row =  The information of :

- Mandatory columns:
  - ***warehouse_id***
  - ***product_id***
  - ***product_stock_procurement_quantity***
  
### Product allocation baseline
With [this dataset](dataset:product_allocation_baseline) contains information about the number of different products shipped from a specific warehouse to a specific store. This data represents baseline product allocation.

**Data format requirement** : 1 row =  The information of :

- Mandatory columns:
  - ***warehouse_id***
  - ***forecast_granularity***
  - ***product_id***
  - ***product_quantity***
  
---


# Data storage requirements
The ***Inventory Allocation Optimization*** Dataiku Application is designed to work with SQL storage.
To benefit natively from all the Dataiku application automation, you will need one of the following connections :
- Snowflake
- Google Cloud Platform:
  - BigQuery + GCS **(⚠️Both are required if you want to leverage redshift)**
- PostgreSQL ** (we used PostgreSQL service on GCP for our solution)**


Roll-out and customization services can be offered on demand for other data storage connections.





